use crate::ast::node::level::ModuleLevelDeclaration;
use crate::ast::node::names::FullyQualifiedName;
use crate::ast::node::use_statement::UseStatement;

#[derive(Debug)]
pub struct CompilationUnit {
    file_name: String,
    file_id: usize,
    namespace: Option<Box<FullyQualifiedName>>,
    use_statements: Vec<Box<UseStatement>>,
    declarations: Vec<Box<ModuleLevelDeclaration>>,
}

impl CompilationUnit {
    pub fn new(
        file_name: String,
        file_id: usize,
        namespace: Option<Box<FullyQualifiedName>>,
        use_statements: Vec<Box<UseStatement>>,
        declarations: Vec<Box<ModuleLevelDeclaration>>,
    ) -> Self {
        Self {
            file_name,
            file_id,
            namespace,
            use_statements,
            declarations,
        }
    }

    pub fn file_name(&self) -> &str {
        &self.file_name
    }

    pub fn file_id(&self) -> usize {
        self.file_id
    }

    pub fn namespace(&self) -> Option<&FullyQualifiedName> {
        // We have to use this non-map syntax because we need a reference like `&self.namespace`
        if let Some(namespace) = &self.namespace {
            Some(namespace.as_ref())
        } else {
            None
        }
    }

    pub fn use_statements(&self) -> Vec<&UseStatement> {
        self.use_statements.iter().map(Box::as_ref).collect()
    }

    pub fn use_statements_mut(&mut self) -> Vec<&mut UseStatement> {
        self.use_statements.iter_mut().map(Box::as_mut).collect()
    }

    pub fn declarations(&self) -> Vec<&ModuleLevelDeclaration> {
        self.declarations.iter().map(Box::as_ref).collect()
    }

    pub fn declarations_mut(&mut self) -> Vec<&mut ModuleLevelDeclaration> {
        self.declarations.iter_mut().map(Box::as_mut).collect()
    }
}
