use crate::ast::node::names::Identifier;
use crate::ast::node::type_use::TypeUse;

#[derive(Debug)]
pub struct GenericArguments(Vec<Box<TypeUse>>);

impl GenericArguments {
    pub fn new(arguments: Vec<Box<TypeUse>>) -> Self {
        Self(arguments)
    }

    pub fn is_empty(&self) -> bool {
        self.0.is_empty()
    }

    pub fn arguments(&self) -> Vec<&TypeUse> {
        self.0.iter().map(Box::as_ref).collect()
    }

    pub fn arguments_mut(&mut self) -> Vec<&mut TypeUse> {
        self.0.iter_mut().map(Box::as_mut).collect()
    }
}

impl Default for GenericArguments {
    fn default() -> Self {
        GenericArguments(Vec::new())
    }
}

#[derive(Debug)]
pub struct GenericParameters(Vec<Box<Identifier>>);

impl GenericParameters {
    pub fn new(identifiers: Vec<Box<Identifier>>) -> Self {
        Self(identifiers)
    }

    pub fn is_empty(&self) -> bool {
        self.0.is_empty()
    }

    pub fn identifiers(&self) -> Vec<&Identifier> {
        self.0.iter().map(Box::as_ref).collect()
    }

    pub fn identifiers_mut(&mut self) -> Vec<&mut Identifier> {
        self.0.iter_mut().map(Box::as_mut).collect()
    }
}

impl Default for GenericParameters {
    fn default() -> Self {
        GenericParameters(Vec::new())
    }
}
